/* program8_1: Adatklds az SPI0 csatornn
 * A program 0-tl 99-ig rja ki a szmokat egy ktjegy kijelzre.
 *
 * Hardver kvetelmnyek: 
 * Kt sorbakttt 74HC595 shift regiszter felhasznlsval
 * kt darab hetszegmenses LED kijelzt vezrlnk. Az ltalunk hasznlt
 * kzs and kijelzk miatt a kimen adatokat komplementlva kell kikldeni.
 *
 * Arduino kompatibilis lbkiosztst hasznlunk:
 * D13 (PTD1): SPI SCK
 * D12 (PTD3): SPI MISO (ebben a programban nem hasznljuk)
 * D11 (PTD2): SPI MOSI
 * D10 (PTD0): SPI SS
 * 
 * A program forrsa: Mazidi et al., Freescale ARM Cortex-M Embedded Programming
 * http://www.microdigitaled.com/ARM/Freescale_ARM/Code/Freescale_ARM_codes.htm
 *
 * A tanknyvi programot alaposan tdolgoztuk, csak tvolrl hasonlt az eredetire...
 * A rendszer rajelek belltsa: CPU 48 MHz, Bus 24 MHz 
 */

#include "MKL25Z4.h"
// Szmjegy kdok 7-szegmenses kijelzhz
const unsigned char digit [10] = {
  0xFC,  // 0b11111100 - 0
  0x60,  // 0b01100000 - 1
  0xDA,  // 0b11011010 - 2
  0xF2,  // 0b11110010 - 3
  0x66,  // 0b01100110 - 4
  0xB6,  // 0b10110110 - 5 
  0xBE,  // 0b10111110 - 6
  0xE0,  // 0b11100000 - 7
  0xFE,  // 0b11111110 - 8
  0xF6}; // 0b11110110 - 9

void SPI0_init(void);
void SPI0_write(unsigned char data);
	
//---------------------------------------
// Kslelteto fggvny 48 MHz rajelhez
//---------------------------------------
void delayMs(int n) {           
    int i, j;
    for(i = 0 ; i < n; i++)
        for (j = 0; j < 8010; j++); 
}
	

int main(void) {
    unsigned char n, d0, d1;   
    SPI0_init();                // Az SPI0 modul konfigurlsa    
    while(1) {
        for(n=0; n<100; n++) {
            d1 = ~digit[n/10];  // Elso szmjegy szegmensei
            d0 = ~digit[n%10];  // Msodik szmjegy szegmensei
					  PTD->PCOR = 1;      // SS aktivlsa
            SPI0_write(d0);     // Egyesek kikldse
            SPI0_write(d1);     // Tzesek kikldse
            PTD->PSOR = 1;      // SS deaktivlsa	
            delayMs(500);					
        }
    }
}

void SPI0_init(void) {
    SIM->SCGC5 |= 0x1000;       // Port D engedlyezse
    PORTD->PCR[1] = 0x200;      // PTD1 legyen SPI SCK */
    PORTD->PCR[2] = 0x200;      // PTD2 legyen SPI MOSI */
    PORTD->PCR[0] = 0x100;      // PTD0 legyen GPIO mdban
    PTD->PDDR |= 0x01;          // PTD0 kimenet legyen (SPI SS)
    PTD->PSOR = 0x01;           // Kezdetben '1' legyen
   
    SIM->SCGC4 |= 0x400000;     // Az SPI0 modul engedlyezse
    SPI0->C1 = 0x10;            // SPI letiltsa, master md
    SPI0->C1 |= 0x01;           // LSBFE = 1 (elszr LSB-t kldjk) 
    SPI0->C2 = 0;               // Alaprtelmezett belltsok
    SPI0->BR = 0x54;            // Baud rate = 125 kHz (/6 s /32 osztk)
    SPI0->C1 |= 0x40;           // Az SPI modul engedlyezse
}

void SPI0_write(unsigned char data) {
    volatile char dummy;    
    while(!(SPI0->S & 0x20)) { }    /* wait until tx ready */
    SPI0->D = data;                 /* send data byte */
    while(!(SPI0->S & 0x80)) { }    /* wait until tx complete */
    dummy = SPI0->D;                /* clear SPRF */    
}

