/* program8_2: 8x8 LED mtrix vezrlse MAX7219 IC-vel
 *
 * A program egy 8x8-as LED mtrix kijelzot vezrel 
 * MAX7219 IC-vel, ami SPI buszon kommunikl
 *
 * Hardver kvetelmnyek: 
 * 8x8 LED mtrix MAX7219 vezrlvel
 *
 * Arduino kompatibilis lbkiosztst hasznlunk:
 * D13 (PTD1): SPI SCK
 * D12 (PTD3): SPI MISO (ebben a programban nem hasznljuk)
 * D11 (PTD2): SPI MOSI
 * D10 (PTD0): SPI SS
 * 
 * A rendszer rajelek belltsa: CPU 48 MHz, Bus 24 MHz 
 */
 
#include "MKL25Z4.h"

void SPI0_init(void);
void max7219_write(unsigned char command, unsigned char data);
void delayMs(int n);

// MAX7219 parancskdok:
#define NO_OP      0
#define DECODE     9
#define INTENSITY 10
#define SCANLIMIT 11
#define SHUTDOWN  12
#define TESTMODE  15

const unsigned char minta1[]= {
    0xFF,0x18,0x18,0x18,0x18,0x18,0x18,0xFF
};  //H
const unsigned char minta2[]= {
    0x1F,0x60,0x80,0x40,0x40,0x80,0x60,0x1F
};  //W

int main(void) {
    unsigned char i;
    SPI0_init();                 // SPI0 konfigurlsa    
    max7219_write(DECODE, 0);    // Nincs dekdols
    max7219_write(SCANLIMIT, 7); // psztzs 8 sorra/oszlopra
    max7219_write(INTENSITY, 8); // Kitltsi arny = 17/32
    max7219_write(TESTMODE, 1);  // Teszt md engedlyezse
    max7219_write(SHUTDOWN, 1);  // Megjelents engedlyezse
    for(i=1; i<9; i++) {
        max7219_write(i,0);      // kppontok trlse
    }
    delayMs(1000);
    max7219_write(TESTMODE, 0);  // Teszt md letiltsa
    delayMs(1000);     

    while(1) {
        for(i=1; i<9; i++) {     // 1. minta kirajzolsa
            max7219_write(i,minta1[i-1]);
        }
        delayMs(1000);
        for(i=1; i<9; i++) {    // 2. minta kirajzolsa    
            max7219_write(i,minta2[i-1]);  
        }
        delayMs(1000);				
    }
}



void max7219_write(unsigned char command, unsigned char data) {
    volatile char dummy;
    
    PTD->PCOR = 1;                  // SS aktivls

    while(!(SPI0->S & 0x20)) { }    // TX ksz jelre vr
    SPI0->D = command;              // Parancs kldse
    while(!(SPI0->S & 0x80)) { }    // tvitel vgre vr
    dummy = SPI0->D;                // SPRF trlse   
    while(!(SPI0->S & 0x20)) { }    // TX ksz jelre vr
    SPI0->D = data;                 // Adat kldse
    while(!(SPI0->S & 0x80)) { }    // tvitel vgre vr
    dummy = SPI0->D;                // SPRF trlse
    
    PTD->PSOR = 1;                  // SS deaktivlsa
}



	
//---------------------------------------
// Kslelteto fggvny 48 MHz rajelhez
//---------------------------------------
void delayMs(int n) {           
    int i, j;
    for(i = 0 ; i < n; i++)
        for (j = 0; j < 8010; j++); 
}
	
//---------------------------------------
// SPI0 modul konfigurlsa
//---------------------------------------
void SPI0_init(void) {
    SIM->SCGC5 |= 0x1000;       // Port D engedlyezse
    PORTD->PCR[1] = 0x200;      // PTD1 legyen SPI SCK */
    PORTD->PCR[2] = 0x200;      // PTD2 legyen SPI MOSI */
    PORTD->PCR[0] = 0x100;      // PTD0 legyen GPIO mdban
    PTD->PDDR |= 0x01;          // PTD0 kimenet legyen (SPI SS)
    PTD->PSOR = 0x01;           // Kezdetben '1' legyen
   
    SIM->SCGC4 |= 0x400000;     // Az SPI0 modul engedlyezse
    SPI0->C1 = 0x10;            // SPI letiltsa, master md, MSB elszr
    SPI0->C2 = 0;               // Alaprtelmezett belltsok
    SPI0->BR = 0x51;            // Baud rate = 1 MHz (/6 s /4 osztk)
    SPI0->C1 |= 0x40;           // Az SPI modul engedlyezse
}


