/* program8_4: Nokia 5110 LCD vezrlse
 *
 * A program egy Nokia 5110 LCD kijelzt vezrel, 
 * ami SPI buszon kommunikl. A kijelz PCD8544
 * vezrlot tartalmaz.
 *
 * Hardver kvetelmnyek: 
 * Nokia5110 kijelz
 *
 * Arduino kompatibilis lbkiosztst hasznlunk:
 * D13 (PTD1): SPI SCK   ===>  LCD SCLK   
 * D12 (PTD3): SPI MISO  nem hasznljuk
 * D11 (PTD2): SPI MOSI  ===>  LCD DIN
 * D10 (PTD0): SPI SS    ===>  LCD CE
 * D9  (PTD5)            ===>  LCD D/C (vagy RS)
 * D2  (PTD4)            ===>  LCD RST (RESET)  
 * 
 * A rendszer rajelek belltsa: CPU 48 MHz, Bus 24 MHz 
 */
 
#include "MKL25Z4.h"
#include "english_6x8_pixel.h"
#include "nxp_logo.h"

void SPI0_init(void);
void delayMs(int n);
void LCD_init(void);
void LCD_clear(void);
void LCD_write_string(unsigned char x,unsigned char y,char *s);
void LCD_write_char(unsigned char c);
void LCD_write_byte(unsigned char data, unsigned char dc);
void LCD_write_logo(unsigned const char* ptr);

int main(void) {
    SPI0_init();                   // SPI0 konfigurlsa    
    LCD_init();  
    while(1) {
      LCD_write_logo(nxp_logo);    // Bitkp megjelents
      delayMs(5000);               // 5 s vrakozs 
      LCD_clear();                 // Kperny trls                
      LCD_write_string(0,0,"Nokia5110 LCD ");
      LCD_write_string(0,1,"driven by SPI0");
      LCD_write_string(0,2,"on FRDM-KL25Z ");
      LCD_write_string(0,3,"--------------");                 
      LCD_write_string(0,4,"(c) I. Cserny,"); 
      LCD_write_string(0,5,"Febr 06, 2017.");                  
      delayMs(5000);                        
    }
}
	
//---------------------------------------
// Kslelteto fggvny 48 MHz rajelhez
//---------------------------------------
void delayMs(int n) {           
    int i, j;
    for(i = 0 ; i < n; i++)
        for (j = 0; j < 8010; j++); 
}
	
//---------------------------------------
// SPI0 modul konfigurlsa
//---------------------------------------
void SPI0_init(void) {
    SIM->SCGC5 |= 0x1000;        // Port D engedlyezse
    PORTD->PCR[0] = 0x100;       // PTD0 legyen GPIO mdban
    PORTD->PCR[1] = 0x200;       // PTD1 legyen SPI SCK */
    PORTD->PCR[2] = 0x200;       // PTD2 legyen SPI MOSI */
    PORTD->PCR[4] = 0x100;       // PTD4 legyen GPIO mdban		
    PORTD->PCR[5] = 0x100;       // PTD5 legyen GPIO mdban	
    PTD->PDDR |= 0x31;           // PTD0, PTD4, PTD5 legyen kimenet
    PTD->PSOR = 0x11;            // PTD0 s PTD4 '1' legyen
   
    SIM->SCGC4 |= 0x400000;      // Az SPI0 modul engedlyezse
    SPI0->C1 = 0x10;             // SPI letiltsa, master md, MSB elszr
    SPI0->C2 = 0;                // Alaprtelmezett belltsok
    SPI0->BR = 0x51;             // Baud rate = 1 MHz (/6 s /4 osztk)
    SPI0->C1 |= 0x40;            // Az SPI modul engedlyezse
}

void LCD_write_byte(unsigned char data, unsigned char dc) {
    unsigned char dummy;
	  PTD->PCOR = 1;               // SS aktivls    
    if(dc) { 
      PTD->PSOR = 0x20;          // D/C = 1
    }
    else {	
      PTD->PCOR = 0x20;          // D/C = 0
    }	    
    while(!(SPI0->S & 0x20)) { } // TX ksz jelre vr
    SPI0->D = data;              // Adat kldse
    while(!(SPI0->S & 0x80)) { } // tvitel vgre vr
    dummy = SPI0->D;             // SPRF trlse   
    PTD->PSOR = 1;               // SS deaktivlsa
  }

void LCD_write_logo(unsigned const char * ptr) {
    unsigned int ctr = 0; 
    while(ctr++ < 504) {
      LCD_write_byte(*ptr++,1);
    }  
}	


/************************************************
 * LCD_clear: Kperny trlse
 ************************************************/
void LCD_clear(void) {
    unsigned int i;
    LCD_write_byte(0x0c, 0);			
    LCD_write_byte(0x80, 0);			
    for (i=0; i<504; i++)
      LCD_write_byte(0, 1);			
}

/*********************************************
 * LCD_set_XY: kurzor belltsa megadott helyre
 * x: karakterhely szma (0-83)
 * y: sor szma (0-5)
 *********************************************/
void LCD_set_XY(unsigned char X, unsigned char Y)
  {
    LCD_write_byte(0x40 | Y, 0); // column
    LCD_write_byte(0x80 | X, 0); // row
  }

void LCD_write_char(unsigned char c) {
    unsigned char line;
    for (line=0; line<6; line++)
      LCD_write_byte(font6x8[c-32][line], 1);
}
	
void LCD_write_string(unsigned char x,unsigned char y,char *s) {
    LCD_set_XY(x,y);
    while (*s) LCD_write_char(*s++);
}
	
void LCD_init(void) {
    PTD->PCOR = 0x10;            // LCD_RST lehzsa
    delayMs(100);
    PTD->PSOR = 0x10;            // LCD_RST felhzsa  
    delayMs(100);	
    LCD_write_byte (0x21, 0);    // use the extended command set the LCD mode
    LCD_write_byte (0xc8, 0);    // set the bias voltage
    LCD_write_byte (0x06, 0);    // temperature correction
    LCD_write_byte (0x13, 0);    // 1:48
    LCD_write_byte (0x20, 0);    // use basic commands
    LCD_clear ();                // clear the screen
    LCD_write_byte (0x0c, 0);    // set display mode, the normal display
  }

